// $Id: OutlinePanel.java 16027 2012-07-13 11:18:01Z mike $

package org.faceless.pdf2.viewer2.feature;

import org.faceless.pdf2.viewer2.*;
import org.faceless.pdf2.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.awt.*;
import java.util.*;
import java.util.List;

/**
 * Create a {@link SidePanel} that will display the document bookmarks, as returned
 * by {@link PDF#getBookmarks}.
 *
 * <span class="featurename">The name of this feature is <span class="featureactualname">ShowHideBookmarks</span></span>
 * <p><i>This code is copyright the Big Faceless Organization. You're welcome to use, modify and distribute it in any form in your own projects, provided those projects continue to make use of the Big Faceless PDF library.</i></p>
 * @since 2.8
 */
public class OutlinePanel extends SidePanelFactory
{
    /**
     * Return an OutlinePanel
     * @deprecated since 2.11 just calls the public constructor
     */
    public OutlinePanel getInstance() {
        return new OutlinePanel();
    }

    /**
     * Create a new OutlinePanel
     * @since 2.11
     */
    public OutlinePanel() {
        super("ShowHideBookmarks");
    }

    public boolean isSidePanelRequired(PDF pdf) {
        return pdf.getBookmarks().size()>0;
    }

    public SidePanel createSidePanel() {
        return new OutlinePanelImpl();
    }
}

/**
 * A {@link SidePanel} representing the "Outline" or "Bookmark" panel.
 *
 * <p><i>
 * This code is copyright the Big Faceless Organization. You're welcome to
 * use, modify and distribute it in any form in your own projects, provided
 * those projects continue to make use of the Big Faceless PDF library.
 * </i></p>
 */
class OutlinePanelImpl extends JPanel implements TreeSelectionListener, SidePanel
{
    private JTree outline;
    private DefaultMutableTreeNode root;
    private JScrollPane scrollpane;
    private DocumentPanel panel;
    private PDF pdf;
    private Icon icon;

    OutlinePanelImpl() {
        super(new BorderLayout());
        setOpaque(true);
        scrollpane = new JScrollPane();
        Util.fixScrollPaneKeyBindings(scrollpane);
        this.icon = new ImageIcon(PDFViewer.class.getResource("resources/icons/bookmark.png"));
        add(scrollpane, BorderLayout.CENTER);
    }

    public Icon getIcon() {
        return icon;
    }

    public String getName() {
        return "Bookmarks";
    }

    public void panelVisible() {
    }

    public void panelHidden() {
    }

    public void setDocumentPanel(DocumentPanel panel) {
        this.panel = panel;
        this.pdf = panel==null ? null : panel.getPDF();
        if (pdf!=null && !pdf.getBookmarks().isEmpty()) {
            root = new DefaultMutableTreeNode(null) {
                public String toString() { return UIManager.getString("PDFViewer.Bookmarks"); }
            };
            outline = new JTree(root);
            outline.setFont(new Font("SansSerif", Font.PLAIN, 10));
            DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer() {
                public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {
                    PDFBookmark bookmark = (PDFBookmark)((DefaultMutableTreeNode)value).getUserObject();
                    if (bookmark!=null) {
                        Color color = bookmark.getColor();
                        boolean bold = bookmark.isBold();
                        boolean italic = bookmark.isItalic();
                        setTextNonSelectionColor(color==null ? Color.black : color);
                        Font font = getFont();
                        setFont(font.deriveFont((bold ? Font.BOLD : 0) + (italic ? Font.ITALIC : 0)));
                    }
                    super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
                    return this;
                }
            };
            // Windows has issues with 8-bit images - use our own.
            if (Util.isLAFWindows()) {
                renderer.setLeafIcon(new ImageIcon(PDFViewer.class.getResource("resources/bookmarkpage.png")));
            }
            outline.setCellRenderer(renderer);
            outline.setEditable(false);
            outline.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
            outline.addTreeSelectionListener(this);

            outline.addMouseMotionListener(new MouseMotionListener() {
                public void mouseMoved(MouseEvent e) {
                    int row = outline.getRowForLocation(e.getX(), e.getY());
                    setCursor(row == -1 ? null : new Cursor(Cursor.HAND_CURSOR));
                }
                public void mouseDragged(MouseEvent e) { }
            });

            scrollpane.setVisible(true);
            ArrayList l = new ArrayList();
            loadOutline(pdf.getBookmarks(), root, l);
            for (int i=0;i<l.size();i++) {
                outline.expandPath((TreePath)l.get(i));
            }
            scrollpane.setViewportView(outline);
            outline.setRootVisible(true);
            outline.expandRow(0);
        } else {
            scrollpane.setVisible(false);
            scrollpane.setViewportView(null);
            root = null;
            outline = null;
        }
        repaint();
    }

    private void loadOutline(List bookmarks, DefaultMutableTreeNode root, Collection open) {
        for (Iterator i = bookmarks.iterator();i.hasNext();) {
            final PDFBookmark bookmark = (PDFBookmark)i.next();
            DefaultMutableTreeNode node = new DefaultMutableTreeNode(bookmark) {
                public String toString() {
                    return bookmark.getName();
                }
            };
            root.add(node);
            List children = bookmark.getBookmarks();
            if (children != null) {
                node.setAllowsChildren(true);
                loadOutline(children, node, open);
            }
            if (bookmark.isOpen()) {
                open.add(new TreePath(node.getPath()));
            }
        }
    }

    public void valueChanged(TreeSelectionEvent e) {
        Object o = outline.getLastSelectedPathComponent();
        if (o instanceof DefaultMutableTreeNode) {
            o = ((DefaultMutableTreeNode)o).getUserObject();
            if (o instanceof PDFBookmark) {
                panel.getJSManager().runEventBookmarkMouseUp(panel, ((PDFBookmark)o));
            }
        }
    }
}
